package edu.nyu.scps.igpay;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.os.Bundle;
import android.speech.RecognizerIntent;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.widget.LinearLayout;
import android.widget.TextView;

public class IgpayActivity extends Activity {
	private static final int VOICE_RECOGNITION_REQUEST_CODE = 1;
	LinearLayout linearLayout;
	TextView textView;
	TextView textView1;
	TextView textView2;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		linearLayout = (LinearLayout)findViewById(R.id.linearLayout);
		textView = (TextView)findViewById(R.id.textView);
		textView1 = (TextView)findViewById(R.id.textView1);
		textView2 = (TextView)findViewById(R.id.textView2);

		//Are there any activities that recognize speech?
		PackageManager packageManager = getPackageManager();
		Intent intent = new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH);
		List<ResolveInfo> activities = packageManager.queryIntentActivities(intent, 0);

		for (int i = 0; i < activities.size(); ++i) {
			final ResolveInfo resolveInfo = activities.get(i);
			final ActivityInfo activityInfo = resolveInfo.activityInfo;
			textView.append("(" + activityInfo.loadLabel(packageManager) + ")\n");
		}

		if (activities.size() == 0) {
			textView.setText("Sorry, no activities recognize speech.");
		} else {
			textView.setOnTouchListener(new OnTouchListener() {
				@Override
				public boolean onTouch(View view, MotionEvent motionEvent) {
					if (motionEvent.getAction() == MotionEvent.ACTION_DOWN) {
						Intent intent = new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH);
						intent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Speak in English");

						intent.putExtra(
							RecognizerIntent.EXTRA_LANGUAGE_MODEL,
							RecognizerIntent.LANGUAGE_MODEL_FREE_FORM
						);

						// How many results you want to receive, sorted
						// so that the first result is the one with higher confidence.
						intent.putExtra(RecognizerIntent.EXTRA_MAX_RESULTS, 1);

						startActivityForResult(intent, VOICE_RECOGNITION_REQUEST_CODE);
						return true;
					}
					return false;
				}
			});
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);

		if (requestCode == VOICE_RECOGNITION_REQUEST_CODE && resultCode == RESULT_OK) {

			// The string the recognizer thought it most likely heard
			ArrayList<String> list = data.getStringArrayListExtra(
				RecognizerIntent.EXTRA_RESULTS);

			if (list.size() > 0) {
				String english = list.get(0);
				String pigLatin = toPigLatin(english);

				//Capitalize the first letter.
				String initial = english.substring(0, 1).toUpperCase();
				english = initial + english.substring(1);
				initial = pigLatin.substring(0, 1).toUpperCase();
				pigLatin = initial + pigLatin.substring(1);

				//Pig Latin is easier to read when printed one word per line.
				english = english.replaceAll(" ", "\n");
				pigLatin = pigLatin.replaceAll(" ", "\n");

				english = english + ".";
				pigLatin = pigLatin + ".";

				textView1.setText(english);
				textView2.setText(pigLatin);
			}
		}
	}

	private String toPigLatin(String s) {
		//apple temporarily becomes wapple (ultimately appleway)
		//\b is a word boundary; backslash in double-quoted string needs a backslash.
		Pattern pattern = Pattern.compile("\\b([aeiou])");
		Matcher matcher = pattern.matcher(s);
		s = matcher.replaceAll("w$1");

		//boy becomes oybay; spring becomes ingspray
		pattern = Pattern.compile("(s[pt]r|sd|[cpst]h|[bcfgps]l|kn|sp|[bcfpt]r|st|zh|[a-z])([a-z]*)");
		matcher = pattern.matcher(s);
		s = matcher.replaceAll("$2$1ay");

		return s;
	}
}